﻿-- Author      : Kurapica
-- Create Date : 6/30/2008

--[[
    functions to build class system
--]]

do
	-- Check Version
	local version = 1
	if not IGAS.Register("IGAS.GUI.Base.Func", version) then
		return
	end
    
    local error = error
    local rawset = rawset
	local rawget = rawget
    local type = type
    local setmetatable = setmetatable
    local unpack = unpack
    
	local _GUI = IGAS.GUI
	local _BaseFunc = IGAS.GUI.Base.Func
	local _Widget = IGAS.GUI.Widget
    local _Enum = IGAS.GUI.Base.Enum
    
 	------------------------------------------------------
	--------------------- Script Handlers -----------------
	------------------------------------------------------
	local _ScriptHandler = {}
	
	-- OnEvent
	_ScriptHandler["OnEvent"] = function (self, _event,...)
		if self.__Wrapper[_event] then
			self.__Wrapper[_event](self.__Wrapper, _event, ...)
		end
        if (self.__Wrapper["OnEvent"] and type(self.__Wrapper["OnEvent"]) == "function") then 
			self.__Wrapper["OnEvent"](self.__Wrapper, _event,...)
		end
	end
	
    -- OnUpdate    
	_ScriptHandler["OnUpdate"] = function (self, ...)
		if (self.__Wrapper["__OnUpdate"] and type(self.__Wrapper["__OnUpdate"]) == "function") then 
			self.__Wrapper["__OnUpdate"](self.__Wrapper, ...)
		end
	end
    
	-- Other Sctipt
	for _, sc in ipairs(_Enum.ScriptType) do
		if not _ScriptHandler[sc] then
			_ScriptHandler[sc] = function(self, ...)
				if (self.__Wrapper[sc] and type(self.__Wrapper[sc]) == "function") then 
					self.__Wrapper[sc](self.__Wrapper, ...)
				end
			end
		end
	end
	
 	------------------------------------------------------
	--------------------- MetaTables ---------------------
	------------------------------------------------------
	local _metaT = {
		__index = function(_table, _key)
            local _Object = _table.__ObjectType
            
			-- Special key: __Childs
			if _key == "__Childs" then
				rawset(_table, _key, {})
				return rawget(_table, _key)
			end
			
			-- Property Get
			if _Object.Property and _Object.Property[_key] then
				if _Object.Property[_key]["Get"] then
					return _Object.Property[_key]["Get"](_table)
				else
					error(_key.." is write-only.",2)
				end
			end
			
			-- Frame Func
			if _Object.FuncProxy and _Object.FuncProxy[_key] then
				return _Object.FuncProxy[_key]
			end
			
			-- Child
			if _table.__Childs and type(_table.__Childs) == "table" and _table.__Childs[_key] then
				return _table.__Childs[_key]
			end
		end,
		
		__newindex = function(_table, _key, _value)
            local _Object = _table.__ObjectType
            
			-- Special key: __Childs
			if _key == "__Childs" and type(_value) ~= "table" then
				error(_key.." must be a table.", 2)
			end
			
			-- Property Set
			if _Object.Property and _Object.Property[_key] then
				if _Object.Property[_key]["Set"] then
					return _Object.Property[_key]["Set"](_table,_value)
				else
					error(_key.." is read-only.", 2)
				end
			end
			
			-- Child
			if _table.__Childs and type(_table.__Childs) == "table" and _table.__Childs[_key] then
				error(_key.." is a child-widget, can't be used in other way.", 2)
			end
			
			-- OnUpdate, reduce the cpu cost.
			if _key == "OnUpdate" and _table["SetScript"] then
                if _value and type(_value) == "function" then
                    _table:SetScript("OnUpdate", _ScriptHandler["OnUpdate"])
                    rawset(_table,"__OnUpdate",_value)
                    return
                else
                    _table:SetScript("OnUpdate", nil)
                    rawset(_table,"__OnUpdate", nil)
                    return
                end
			end
			
			rawset(_table,_key,_value)			-- Other key can be set as usual	
		end,
	}

 	------------------------------------------------------
	------------------- Base Functions -------------------
	------------------------------------------------------    
    
    _BaseFunc.Wrapper = function(object, objectType)
        if not object or type(object) ~= "table" then
            return nil
        end
        
        local _UI, _Wrapper, _Type, _objectType
        
        -- Set _UI, _Wrapper
        if object.__UI or object.__Wrapper then
            if object.__UI then
                _UI = object.__UI
                _Wrapper = object
            elseif object.__Wrapper then
                _UI = object
                _Wrapper = object.__Wrapper
            end
        else
            _UI = object
            _Wrapper = {}
        end
        
        -- Check objectType
        _Type = nil
        
        _objectType = objectType or (_Wrapper["GetObjectType"] and _Wrapper:GetObjectType()) or (_UI["GetObjectType"] and _UI:GetObjectType()) or "UIObject"
        
        if _objectType then
			if type(_objectType) == "string" and _Widget[_objectType] then
				_Type = _Widget[_objectType]
			elseif type(_objectType) == "table" and _objectType["FuncProxy"] then
				_Type = _objectType
			end
		end
        
        if not _Type then
        	error("no such objectType")
       	end
        
        _UI.__Wrapper = _Wrapper
        _Wrapper.__UI = _UI
        
        _Wrapper.__ObjectType = _Type
        
        setmetatable(_Wrapper, _metaT)
        
		-- Script 
		if _Wrapper["HasScript"] and _Wrapper["SetScript"] then
			for k, f in pairs(_ScriptHandler) do
				if k ~= "OnUpdate" and _Wrapper:HasScript(k) and (not _Wrapper:GetScript(k)) then
                    _Wrapper:SetScript(k, f)
				end
			end
		end
		
        return _Wrapper
    end
    
	_BaseFunc.GetUI = function(ob)
	    return ob and (ob.__UI or ob)
	end
	
	_BaseFunc.GetWrapper = function(ob)
        if ob.__Wrapper then
            return ob.__Wrapper
		elseif not ob["IsIGASWidget"] then
			return _BaseFunc.Wrapper(ob)
        else
            return ob
		end
	end
    
    _BaseFunc.CallMethod = function(_superType, _method, ob, ...)
        if _superType["FuncProxy"][_method] then
        	_superType["FuncProxy"][_method](ob, ...)
        end
    end
 	------------------------------------------------------
	------------------ Argument Convert ------------------
	------------------------------------------------------    
    local function GetRet(arrV, i, maxI)
        if i < maxI then
            return arrV[i], GetRet(arrV, i+1, maxI)
        else
            return arrV[i]
        end
    end
    
    _BaseFunc.GetArgs = function(...)
        local ret = {}
        local e
        
        if select('#', ...) == 0 then
            return nil
        end
        
        for i = 1, select('#', ...) do
            e = select(i, ...)
            if e and type(e) == "table" then
                e = _BaseFunc.GetUI(e)
            end
            
            ret[i] = e
        end
        
        return GetRet(ret, 1, select('#', ...))
    end
    
    _BaseFunc.GetVals = function(...)
        local ret = {}
        local e
        
        if select('#', ...) == 0 then
            return nil
        end
        
        for i = 1, select('#', ...) do
            e = select(i, ...)
            if e and type(e) == "table" and e["GetObjectType"] then
                e = _BaseFunc.GetWrapper(e)
            end
            
            ret[i] = e
        end
        
        return GetRet(ret, 1, select('#', ...))
    end
    
	------------------------------------------------------
	--------------------- Constructor --------------------
	------------------------------------------------------    
    --- Name Creator
	local function NewName(_typeN, _parent)
		local i = 1
		local name = _typeN["Name"]
		local parentF = _parent or UIParent
		
		if not name or name == "" then
			name = "Widget"
		end
		
		parentF = _BaseFunc.GetWrapper(parentF)
		
		while true do
			if parentF:GetChild(name..i) then
				i = i + 1
			else
				break
			end
		end
		
		return name..i
	end
	
	local function NewObject(self, _name, _parent,...)
		local parentF = _parent or UIParent
        
        parentF = _BaseFunc.GetWrapper(parentF)
		
        if _name and type(_name) ~= "string" then
            error("name must be string.")
        end
		
		if not parentF["AddChild"] then
			error("Can't add child to this object.")
		end
		
        if _name and _name ~= "" then            
            -- Check the parent's childs
            if parentF["GetChild"] and parentF:GetChild(_name) then
            	if parentF:GetChild(_name)["GetObjectType"] and parentF:GetChild(_name):GetObjectType() == self then
            		return parentF:GetChild(_name)
            	else
            		error("this name is used")
            	end
            end
        else
        	_name = NewName(self, parentF)
        end
		
		if _name then
			-- New Frame
			local frame = self.Constructor(parentF,...)
			
			if not frame then
				return nil
			end
	        
	      	frame = _BaseFunc.Wrapper(frame, self)
	        
            frame.Name = _name
            parentF:AddChild(frame)
            
			return frame
		else
			return nil
		end
	end
    
    ------------------------------------------------------
	------------------ Global Function -------------------
	------------------------------------------------------    
	--[[
		CreateFrame
		Args:
		     frameType		string or FrameType,like "Frame" or IGAS.GUI.Frame
		     frameName		the object's name
		     parentFrame	the object's parent
	--]]
	_BaseFunc.CreateFrame = function(frameType,frameName,parentFrame,...)
		local _frameType
		
		if (type(frameType) == "string") and (not _Widget[frameType]) then			
			error("invalid frameType")
		end
		
		if type(frameType) == "string" then
			_frameType = _Widget[frameType]
		elseif type(frameType) == "table" then
			_frameType = nil
            if frameType["FuncProxy"] then                
				_frameType = frameType
            end            
			if not _frameType then
				error("invalid frameType")
			end
		else
			error("invalid frameType")
		end
		
        if _frameType.New then
            return _frameType:New(frameName, parentFrame,...)
        else
            error("this frameType is abstracted")
        end
	end
	
	--[[
		New Widget, Used to create Widget's Constructor
		Args:
			 _WidgetInfo				Table		Contains the information about the widget
				 {				
					WidgetName				String		Widget Name
                    Base                    String      Base Widget
					ScriptType	            Table		ScriptType, used for design
					FuncProxy       		Table		FuncProxy
					Property		        Table		Property
					New						Function	Constructor
                    NotChild                Boolean     Can't be added as a child if true
				}
	--]]
	_BaseFunc.NewWidget = function(_WidgetInfo)
	    local _Base = _WidgetInfo.Base
		local _FuncProxy = _WidgetInfo.FuncProxy or {}
		local _Property = _WidgetInfo.Property or {}
		local _ScriptType = _WidgetInfo.ScriptType or {}
        local _NotChild = _WidgetInfo.NotChild or false
		
		-- Inherit from base
        if _Base then
            if type(_Base) == "string" then
                if _Widget[_Base] then
                    _Base = _Widget[_Base]
                else
                    _Base = _Widget["UIObject"]
                end
            elseif type(_Base) == "table" then
                if not _Base["FuncProxy"] then
                    _Base = _Widget["UIObject"]
                end
            else
                _Base = _Widget["UIObject"]
            end
            local _metaFunc = {__index = _Base.FuncProxy}
            local _metaPro = {__index = _Base.Property}
            setmetatable(_FuncProxy, _metaFunc)
            setmetatable(_Property, _metaPro)
        end
		
		--[[
			Frame's metatable
		]]--
		local _NewWidget = {}
        
        local index = {
        	["Name"] = _WidgetInfo.WidgetName,
			["FuncProxy"] = _FuncProxy,
			["Property"] = _Property,
			["ScriptType"] = _ScriptType,
        }
        
		if _NotChild then
			index["NotChild"] = _NotChild
		end
		
        if _Base then
            index["Base"] = _Base
        end
        
        if _WidgetInfo.New and type(_WidgetInfo.New) == "function" then
            index["Constructor"] = _WidgetInfo.New
            index["New"] = NewObject
        end
        
		-- The other parameters will be convert to widget directly.
		for i, v in pairs(_WidgetInfo) do
			if i ~= "WidgetName" and i ~= "FuncProxy" and i ~= "Property" and i ~= "ScriptType" and i ~= "New" and i ~= "NotChild" and i ~= "Base" then
				index[i] = v
			end
		end
		
		local _metaF = {
			__index = index,
		
			__newindex = function(_table,_key,_value)
				error("This object is read-only")
			end,
		}
	    
		setmetatable(_NewWidget, _metaF)
        
        return _NewWidget
	end
    
    ------------------------------------------------------
	-------------------- GUI Function --------------------
	------------------------------------------------------
    _GUI.CreateFrame = _BaseFunc.CreateFrame
    _GUI.NewWidget = _BaseFunc.NewWidget
end