﻿-- Author      : Kurapica
-- Create Date : 7/16/2008 15:01
     
-- CheckButton
do
	-- Check Version
	local version = 1
	if not IGAS.Register("IGAS.GUI.Widget.CheckButton", version) then
		return
	end

    local getmetatable = getmetatable
    local pairs = pairs
    local ipairs = ipairs
    local type = type
    local error = error
    local CreateFrame = CreateFrame
    
	local _WidgetName = "CheckButton"
    local _Base = "Button"

	local _GUI = IGAS.GUI
	local _Widget = IGAS.GUI.Widget
    local _BaseFunc = IGAS.GUI.Base.Func
	local _PropertyType = IGAS.GUI.Base.PropertyType   
	
	-- ScriptType
	local _ScriptType = {
	}
    
	-- FuncProxy
	local _FuncProxy = {
        ["Enable"] = function(self)
            self.__UI:Enable()
            self:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check.blp")
        end,
        ["Disable"] = function(self)
            self.__UI:Disable()
            self:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled.blp")
        end,
    }
    
	-- Property
	local _Property = {
        -- Checked
        ["Checked"] = {
			["Set"] = function(self, value)
				self.__UI:SetChecked(value)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetChecked()
			end,
			
			["Type"] = _PropertyType.Boolean,
		},
    
        -- CheckedTexture
        ["CheckedTexture"] = {
			["Set"] = function(self, texture)
				self.__UI:SetCheckedTexture(texture)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetCheckedTexture()
			end,
			
			["Type"] = _PropertyType.Texture,
		},
    
        -- DisabledCheckedTexture
        ["DisabledCheckedTexture"] = {
			["Set"] = function(self, texture)
				self.__UI:SetDisabledCheckedTexture(texture)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetDisabledCheckedTexture()
			end,
			
			["Type"] = _PropertyType.Texture,
		},
    }
	
    -- New      
    local _New = function(_parent,inheritFrame)
        -- New Frame
		local frame = CreateFrame(_WidgetName,nil,_BaseFunc.GetUI(_parent),inheritFrame)
        
        frame:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up.blp")
        frame:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down.blp")
        frame:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight.blp", "ADD")
        frame:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check.blp")
        frame:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check.blp")
        --frame:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled.blp")
        
        frame:SetWidth(24)
		frame:SetHeight(24)
        
        return frame
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
        ["New"] = _New,
    }
    
    _Widget[_WidgetName] = _GUI.NewWidget(_WidgetInfo)
    
	------------------------------------------------------
	--------------- Scan new function --------------------
    ------------------------------------------------------
	local _frame = CreateFrame(_WidgetName,nil,UIParent)
	
	local _oriMeta = getmetatable(_frame)
	_oriMeta = _oriMeta.__index
    
    local _func = _Widget[_WidgetName]["FuncProxy"]
	-- Other Function
	for _f in pairs(_oriMeta) do
		if not _func[_f] and type(_oriMeta[_f]) == "function" then
			_func[_f] = function(self, ...)
                return _BaseFunc.GetVals(self.__UI[_f](self.__UI, _BaseFunc.GetArgs(...)))
			end
		end
	end
	
	-- Clear
	_oriMeta = nil
    _frame:Hide()
	_frame = nil
end